@@ -1,12 +1,13 @@
 /*
- * Copyright 1999-2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- * 
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -20,7 +21,10 @@ import java.sql.Connection;
 import java.sql.Statement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
-import org.apache.commons.pool.*;
+import org.apache.commons.pool.KeyedObjectPool;
+import org.apache.commons.pool.KeyedObjectPoolFactory;
+import org.apache.commons.pool.PoolableObjectFactory;
+import org.apache.commons.pool.ObjectPool;
 
 /**
  * A {@link PoolableObjectFactory} that creates
@@ -30,7 +34,7 @@ import org.apache.commons.pool.*;
  * @author Glenn L. Nielsen
  * @author James House
  * @author Dirk Verbeeck
- * @version $Revision: 1.23 $ $Date: 2004/06/09 18:21:23 $
+ * @version $Revision: 479187 $ $Date: 2006-11-25 13:13:45 -0700 (Sat, 25 Nov 2006) $
  */
 public class PoolableConnectionFactory implements PoolableObjectFactory {
     /**
@@ -92,7 +96,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
         boolean defaultReadOnly,
         boolean defaultAutoCommit,
         AbandonedConfig config) {
-            
+
         _connFactory = connFactory;
         _pool = pool;
         _config = config;
@@ -124,7 +128,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
         boolean defaultAutoCommit,
         int defaultTransactionIsolation,
         AbandonedConfig config) {
-            
+
         _connFactory = connFactory;
         _pool = pool;
         _config = config;
@@ -159,7 +163,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
         int defaultTransactionIsolation,
         String defaultCatalog,
         AbandonedConfig config) {
-            
+
         _connFactory = connFactory;
         _pool = pool;
         _config = config;
@@ -194,7 +198,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
         int defaultTransactionIsolation,
         String defaultCatalog,
         AbandonedConfig config) {
-            
+
         _connFactory = connFactory;
         _pool = pool;
         _config = config;
@@ -285,7 +289,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
     public void setDefaultCatalog(String defaultCatalog) {
         _defaultCatalog = defaultCatalog;
     }
-    
+
     synchronized public Object makeObject() throws Exception {
         Connection conn = _connFactory.createConnection();
         if(null != _stmtPoolFactory) {
@@ -309,7 +313,7 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
                 return true;
             } catch(Exception e) {
                 return false;
-            }           
+            }
         } else {
             return false;
         }
@@ -330,17 +334,20 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
                     throw new SQLException("validationQuery didn't return a row");
                 }
             } finally {
-                try {
-                    rset.close();
-                } catch(Exception t) {
-                    // ignored
+                if (rset != null) {
+                    try {
+                        rset.close();
+                    } catch(Exception t) {
+                        // ignored
+                    }
                 }
-                try {
-                    stmt.close();
-                } catch(Exception t) {
-                    // ignored
+                if (stmt != null) {
+                    try {
+                        stmt.close();
+                    } catch(Exception t) {
+                        // ignored
+                    }
                 }
-
             }
         }
     }
@@ -352,7 +359,9 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
                 conn.rollback();
             }
             conn.clearWarnings();
-            conn.setAutoCommit(true);
+            if(!conn.getAutoCommit()) {
+                conn.setAutoCommit(true);
+            }
         }
         if(obj instanceof DelegatingConnection) {
             ((DelegatingConnection)obj).passivate();
@@ -365,14 +374,20 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
         }
         if(obj instanceof Connection) {
             Connection conn = (Connection)obj;
-            conn.setAutoCommit(_defaultAutoCommit);
-            if (_defaultTransactionIsolation != UNKNOWN_TRANSACTIONISOLATION) {
+            if (conn.getAutoCommit() != _defaultAutoCommit) {
+                conn.setAutoCommit(_defaultAutoCommit);
+            }
+            if ((_defaultTransactionIsolation != UNKNOWN_TRANSACTIONISOLATION) 
+                    && (conn.getTransactionIsolation() != 
+                    _defaultTransactionIsolation)) {
                 conn.setTransactionIsolation(_defaultTransactionIsolation);
             }
-            if (_defaultReadOnly != null) {
+            if ((_defaultReadOnly != null) && 
+                    (conn.isReadOnly() != _defaultReadOnly.booleanValue())) {
                 conn.setReadOnly(_defaultReadOnly.booleanValue());
             }
-            if (_defaultCatalog != null) {
+            if ((_defaultCatalog != null) &&
+                    (!_defaultCatalog.equals(conn.getCatalog()))) {
                 conn.setCatalog(_defaultCatalog);
             }
         }
@@ -386,14 +401,14 @@ public class PoolableConnectionFactory implements PoolableObjectFactory {
     protected boolean _defaultAutoCommit = true;
     protected int _defaultTransactionIsolation = UNKNOWN_TRANSACTIONISOLATION;
     protected String _defaultCatalog;
-    
+
     /**
      * @deprecated AbandonedConfig is now deprecated.
      */
     protected AbandonedConfig _config = null;
 
     /**
-     * Internal constant to indicate the level is not set. 
+     * Internal constant to indicate the level is not set.
      */
-	static final int UNKNOWN_TRANSACTIONISOLATION = -1;
+    static final int UNKNOWN_TRANSACTIONISOLATION = -1;
 }
