@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -80,7 +81,7 @@ import org.apache.commons.pool.impl.GenericKeyedObjectPool;
  * </p>
  *
  * @author John D. McNally
- * @version $Revision: 1.8 $ $Date: 2004/02/28 12:18:17 $
+ * @version $Revision: 500687 $ $Date: 2007-01-27 16:33:47 -0700 (Sat, 27 Jan 2007) $
  */
 public class DriverAdapterCPDS
     implements ConnectionPoolDataSource, Referenceable, Serializable, 
@@ -113,6 +114,7 @@ public class DriverAdapterCPDS
     private int _timeBetweenEvictionRunsMillis = -1;
     private int _numTestsPerEvictionRun = -1;
     private int _minEvictableIdleTimeMillis = -1;
+    private int _maxPreparedStatements = -1;
 
     private boolean getConnectionCalled = false;
 
@@ -147,13 +149,28 @@ public class DriverAdapterCPDS
         */
         KeyedObjectPool stmtPool = null;
         if (isPoolPreparedStatements()) {
-            stmtPool = new GenericKeyedObjectPool(null,
-                getMaxActive(), GenericKeyedObjectPool.WHEN_EXHAUSTED_GROW, 0, 
-                getMaxIdle(), false, false, getTimeBetweenEvictionRunsMillis(),
-                getNumTestsPerEvictionRun(), 
-                getMinEvictableIdleTimeMillis(), false);            
+            if (getMaxPreparedStatements() <= 0)
+            {
+                // since there is no limit, create a prepared statement pool with an eviction thread
+                //  evictor settings are the same as the connection pool settings.
+                stmtPool = new GenericKeyedObjectPool(null,
+                    getMaxActive(), GenericKeyedObjectPool.WHEN_EXHAUSTED_GROW, 0,
+                    getMaxIdle(), false, false,
+                    getTimeBetweenEvictionRunsMillis(),getNumTestsPerEvictionRun(),getMinEvictableIdleTimeMillis(),
+                    false);
+            }
+            else
+            {
+                // since there is limit, create a prepared statement pool without an eviction thread
+                //  pool has LRU functionality so when the limit is reached, 15% of the pool is cleared.
+                // see org.apache.commons.pool.impl.GenericKeyedObjectPool.clearOldest method
+                stmtPool = new GenericKeyedObjectPool(null,
+                    getMaxActive(), GenericKeyedObjectPool.WHEN_EXHAUSTED_GROW, 0,
+                    getMaxIdle(), getMaxPreparedStatements(), false, false,
+                    -1,0,0, // -1 tells the pool that there should be no eviction thread.
+                    false);
+            }
         }
-        
         // Workaround for buggy WebLogic 5.1 classloader - ignore the
         // exception upon first invocation.
         try {
@@ -201,6 +218,8 @@ public class DriverAdapterCPDS
             String.valueOf(getNumTestsPerEvictionRun())));
         ref.add(new StringRefAddr("minEvictableIdleTimeMillis", 
             String.valueOf(getMinEvictableIdleTimeMillis())));
+        ref.add(new StringRefAddr("maxPreparedStatements",
+            String.valueOf(getMaxPreparedStatements())));
 
         return ref;
     }
@@ -275,6 +294,11 @@ public class DriverAdapterCPDS
                     setMinEvictableIdleTimeMillis(
                         Integer.parseInt(ra.getContent().toString()));
                 }
+                ra = ref.get("maxPreparedStatements");
+                if (ra != null && ra.getContent() != null) {
+                    setMaxPreparedStatements(
+                        Integer.parseInt(ra.getContent().toString()));
+                }
 
                 cpds = this;
             }
@@ -442,7 +466,7 @@ public class DriverAdapterCPDS
 
     /**
      * The maximum number of active statements that can be allocated from
-     * this pool at the same time, or zero for no limit.
+     * this pool at the same time, or non-positive for no limit.
      */
     public int getMaxActive() {
         return (this.maxActive);
@@ -450,7 +474,7 @@ public class DriverAdapterCPDS
 
     /**
      * The maximum number of active statements that can be allocated from
-     * this pool at the same time, or zero for no limit.
+     * this pool at the same time, or non-positive for no limit.
      */
     public void setMaxActive(int maxActive) {
         assertInitializationAllowed();
@@ -459,7 +483,7 @@ public class DriverAdapterCPDS
 
     /**
      * The maximum number of statements that can remain idle in the
-     * pool, without extra ones being released, or zero for no limit.
+     * pool, without extra ones being released, or negative for no limit.
      */
     public int getMaxIdle() {
         return (this.maxIdle);
@@ -467,7 +491,7 @@ public class DriverAdapterCPDS
 
     /**
      * The maximum number of statements that can remain idle in the
-     * pool, without extra ones being released, or zero for no limit.
+     * pool, without extra ones being released, or negative for no limit.
      */
     public void setMaxIdle(int maxIdle) {
         assertInitializationAllowed();
@@ -553,4 +577,27 @@ public class DriverAdapterCPDS
         assertInitializationAllowed();
         _minEvictableIdleTimeMillis = minEvictableIdleTimeMillis;
     }
+    
+    /**
+     * Returns the maximun number of prepared statements.
+     * 
+     * @return maxPrepartedStatements value
+     * @since 1.2.2
+     */
+    public int getMaxPreparedStatements()
+    {
+        return _maxPreparedStatements;
+    }
+
+    /**
+     * Sets the maximum number of prepared statements.
+     * @param maxPreparedStatements the new maximum number of prepared 
+     * statements
+     * 
+     * @since 1.2.2
+     */
+    public void setMaxPreparedStatements(int maxPreparedStatements)
+    {
+        _maxPreparedStatements = maxPreparedStatements;
+    }
 }
