@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -79,7 +80,7 @@ import org.apache.commons.pool.impl.GenericObjectPool;
  * </p>
  *
  * @author John D. McNally
- * @version $Revision: 1.11 $ $Date: 2004/02/28 12:18:17 $
+ * @version $Revision: 500687 $ $Date: 2007-01-27 16:33:47 -0700 (Sat, 27 Jan 2007) $
  */
 public abstract class InstanceKeyDataSource
         implements DataSource, Referenceable, Serializable {
@@ -125,6 +126,7 @@ public abstract class InstanceKeyDataSource
              GenericObjectPool.DEFAULT_MIN_EVICTABLE_IDLE_TIME_MILLIS);
     private boolean _testWhileIdle = GenericObjectPool.DEFAULT_TEST_WHILE_IDLE;
     private String validationQuery = null;
+    private boolean rollbackAfterValidation = false;
     private boolean testPositionSet = false;
 
     protected String instanceKey = null;
@@ -344,11 +346,12 @@ public abstract class InstanceKeyDataSource
     }
     
     /**
-     * Set the value of jndiEnvironment which is used when instantiating
-     * a jndi InitialContext.  This InitialContext is used to locate the
-     * backend ConnectionPoolDataSource.
-     *
-     * @param v  Value to assign to jndiEnvironment.
+     * Sets the value of the given JNDI environment property to be used when
+     * instantiating a JNDI InitialContext. This InitialContext is used to
+     * locate the backend ConnectionPoolDataSource.
+     * 
+     * @param key the JNDI environment property to set.
+     * @param value the value assigned to specified JNDI environment property.
      */
     public void setJndiEnvironment(String key, String value) {
         if (jndiEnvironment == null) {
@@ -419,7 +422,9 @@ public abstract class InstanceKeyDataSource
      * before being returned by the {*link #borrowObject}
      * method.  If the object fails to validate,
      * it will be dropped from the pool, and we will attempt
-     * to borrow another.
+     * to borrow another. For a <code>true</code> value to have any effect,
+     * the <code>validationQuery</code> property must be set to a non-null
+     * string.
      *
      * @see #getTestOnBorrow
      */
@@ -452,7 +457,9 @@ public abstract class InstanceKeyDataSource
      * When <tt>true</tt>, objects will be
      * {*link PoolableObjectFactory#validateObject validated}
      * before being returned to the pool within the
-     * {*link #returnObject}.
+     * {*link #returnObject}. For a <code>true</code> value to have any effect,
+     * the <code>validationQuery</code> property must be set to a non-null
+     * string.
      *
      * @see #getTestOnReturn
      */
@@ -566,7 +573,10 @@ public abstract class InstanceKeyDataSource
      * When <tt>true</tt>, objects will be
      * {*link PoolableObjectFactory#validateObject validated}
      * by the idle object evictor (if any).  If an object
-     * fails to validate, it will be dropped from the pool.
+     * fails to validate, it will be dropped from the pool. For a
+     * <code>true</code> value to have any effect,
+     * the <code>validationQuery</code> property must be set to a non-null
+     * string.
      *
      * @see #getTestWhileIdle
      * @see #setTimeBetweenEvictionRunsMillis
@@ -602,6 +612,34 @@ public abstract class InstanceKeyDataSource
         }
     }
 
+    /**
+     * Whether a rollback will be issued after executing the SQL query 
+     * that will be used to validate connections from this pool
+     * before returning them to the caller.
+     * 
+     * @return true if a rollback will be issued after executing the
+     * validation query
+     * @since 1.2.2
+     */
+    public boolean isRollbackAfterValidation() {
+        return (this.rollbackAfterValidation);
+    }
+
+    /**
+     * Whether a rollback will be issued after executing the SQL query 
+     * that will be used to validate connections from this pool
+     * before returning them to the caller. Default behavior is NOT
+     * to issue a rollback. The setting will only have an effect
+     * if a validation query is set
+     * 
+     * @param rollbackAfterValidation new property value
+     * @since 1.2.2
+     */
+    public void setRollbackAfterValidation(boolean rollbackAfterValidation) {
+        assertInitializationAllowed();
+        this.rollbackAfterValidation = rollbackAfterValidation;
+    }
+
     // ----------------------------------------------------------------------
     // Instrumentation Methods
 
@@ -742,12 +780,23 @@ public abstract class InstanceKeyDataSource
     // Referenceable implementation 
 
     /**
-     * <CODE>Referenceable</CODE> implementation prepares object for
-     * binding in jndi.
+     * Retrieves the Reference of this object.
+     * <strong>Note:</strong> <code>InstanceKeyDataSource</code> subclasses
+     * should override this method. The implementaion included below
+     * is not robust and will be removed at the next major version DBCP
+     * release.
+     *
+     * @return The non-null Reference of this object.
+     * @exception NamingException If a naming exception was encountered
+     *      while retrieving the reference.
      */
+    // TODO: Remove the implementation of this method at next major
+    // version release.
+    
     public Reference getReference() throws NamingException {
-        Reference ref = new Reference(getClass().getName(), 
-            InstanceKeyObjectFactory.class.getName(), null);
+        final String className = getClass().getName();
+        final String factoryName = className + "Factory"; // XXX: not robust 
+        Reference ref = new Reference(className, factoryName, null);
         ref.add(new StringRefAddr("instanceKey", instanceKey));
         return ref;
     }
