@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -25,6 +26,8 @@ import java.util.Iterator;
 import java.util.Map;
 
 import javax.naming.NamingException;
+import javax.naming.Reference;
+import javax.naming.StringRefAddr;
 import javax.sql.ConnectionPoolDataSource;
 
 import org.apache.commons.pool.ObjectPool;
@@ -44,7 +47,7 @@ import org.apache.commons.dbcp.SQLNestedException;
  * </p>
  *
  * @author John D. McNally
- * @version $Revision: 1.10 $ $Date: 2004/02/28 12:18:17 $
+ * @version $Revision: 500687 $ $Date: 2007-01-27 16:33:47 -0700 (Sat, 27 Jan 2007) $
  */
 public class PerUserPoolDataSource
     extends InstanceKeyDataSource {
@@ -95,9 +98,9 @@ public class PerUserPoolDataSource
 
     /**
      * The maximum number of active connections that can be allocated from
-     * this pool at the same time, or zero for no limit.
+     * this pool at the same time, or non-positive for no limit.
      * This value is used for any username which is not specified
-     * in perUserMaxConnections.  The default is 0.
+     * in perUserMaxConnections.
      */
     public int getDefaultMaxActive() {
         return (this.defaultMaxActive);
@@ -105,9 +108,9 @@ public class PerUserPoolDataSource
 
     /**
      * The maximum number of active connections that can be allocated from
-     * this pool at the same time, or zero for no limit.
+     * this pool at the same time, or non-positive for no limit.
      * This value is used for any username which is not specified
-     * in perUserMaxConnections.  The default is 0.
+     * in perUserMaxConnections.  The default is 8.
      */
     public void setDefaultMaxActive(int maxActive) {
         assertInitializationAllowed();
@@ -116,9 +119,9 @@ public class PerUserPoolDataSource
 
     /**
      * The maximum number of active connections that can remain idle in the
-     * pool, without extra ones being released, or zero for no limit.
+     * pool, without extra ones being released, or negative for no limit.
      * This value is used for any username which is not specified
-     * in perUserMaxIdle.  The default is 0.
+     * in perUserMaxIdle.
      */
     public int getDefaultMaxIdle() {
         return (this.defaultMaxIdle);
@@ -126,9 +129,9 @@ public class PerUserPoolDataSource
 
     /**
      * The maximum number of active connections that can remain idle in the
-     * pool, without extra ones being released, or zero for no limit.
+     * pool, without extra ones being released, or negative for no limit.
      * This value is used for any username which is not specified
-     * in perUserMaxIdle.  The default is 0.
+     * in perUserMaxIdle.  The default is 8.
      */
     public void setDefaultMaxIdle(int defaultMaxIdle) {
         assertInitializationAllowed();
@@ -211,7 +214,7 @@ public class PerUserPoolDataSource
 
     /**
      * The maximum number of active connections that can be allocated from
-     * this pool at the same time, or zero for no limit.
+     * this pool at the same time, or non-positive for no limit.
      * The keys are usernames and the value is the maximum connections.  Any 
      * username specified here will override the value of defaultMaxActive.
      */
@@ -225,7 +228,7 @@ public class PerUserPoolDataSource
     
     /**
      * The maximum number of active connections that can be allocated from
-     * this pool at the same time, or zero for no limit.
+     * this pool at the same time, or non-positive for no limit.
      * The keys are usernames and the value is the maximum connections.  Any 
      * username specified here will override the value of defaultMaxActive.
      */
@@ -240,7 +243,7 @@ public class PerUserPoolDataSource
 
     /**
      * The maximum number of active connections that can remain idle in the
-     * pool, without extra ones being released, or zero for no limit.
+     * pool, without extra ones being released, or negative for no limit.
      * The keys are usernames and the value is the maximum connections.  Any 
      * username specified here will override the value of defaultMaxIdle.
      */
@@ -254,7 +257,7 @@ public class PerUserPoolDataSource
     
     /**
      * The maximum number of active connections that can remain idle in the
-     * pool, without extra ones being released, or zero for no limit.
+     * pool, without extra ones being released, or negative for no limit.
      * The keys are usernames and the value is the maximum connections.  Any 
      * username specified here will override the value of defaultMaxIdle.
      */
@@ -413,10 +416,22 @@ public class PerUserPoolDataSource
         }
 
         con.setAutoCommit(defaultAutoCommit);
-        con.setReadOnly(defaultReadOnly);
         if (defaultTransactionIsolation != UNKNOWN_TRANSACTIONISOLATION) {
             con.setTransactionIsolation(defaultTransactionIsolation);
         }
+        con.setReadOnly(defaultReadOnly);
+    }
+
+    /**
+     * Returns a <code>PerUserPoolDataSource</code> {@link Reference}.
+     * 
+     * @since 1.2.2
+     */
+    public Reference getReference() throws NamingException {
+        Reference ref = new Reference(getClass().getName(),
+                PerUserPoolDataSourceFactory.class.getName(), null);
+        ref.add(new StringRefAddr("instanceKey", instanceKey));
+        return ref;
     }
 
     private PoolKey getPoolKey(String username) {
@@ -472,6 +487,7 @@ public class PerUserPoolDataSource
         // the factory with the pool, so we do not have to do so
         // explicitly)
         new CPDSConnectionFactory(cpds, pool, getValidationQuery(),
+                                  isRollbackAfterValidation(), 
                                   username, password);
            
         pools.put(getPoolKey(username), pool);
